// **********************************************************************
//
// Copyright (c) 1999
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1999
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef COS_PROPERTY_IMPL_H
#define COS_PROPERTY_IMPL_H

#include <OB/CosProperty_skel.h>

namespace CosPropertyService
{

class PropertySetDef_impl
    : virtual public POA_CosPropertyService::PropertySetDef
{
    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // My PropertyDefs
    //
    PropertyDefs propertySeq_;

    //
    // List of allowed TypeCodes
    //
    PropertyTypes allowedTypes_;

    //
    // List of allowed properties
    //
    PropertyDefs allowedProperties_;

    //
    // Check name, type and property
    //
    void check(const char*, const CORBA::Any&, PropertyModeType);

    //
    // Find property by name
    //
    PropertyDef* get(const char*, CORBA::ULong* = 0);

public:

    PropertySetDef_impl(PortableServer::POA_ptr);
    PropertySetDef_impl(PortableServer::POA_ptr, const PropertyTypes&,
			const PropertyDefs&);
    PropertySetDef_impl(PortableServer::POA_ptr, const PropertyDefs&);

    PortableServer::POA_ptr _default_POA()
	{ return PortableServer::POA::_duplicate(poa_); }

    //
    // Standard IDL to C++ Mapping
    //
    virtual void define_property(const char*, const CORBA::Any&)
        throw(CosPropertyService::InvalidPropertyName,
              CosPropertyService::ConflictingProperty,
              CosPropertyService::UnsupportedTypeCode,
              CosPropertyService::UnsupportedProperty,
              CosPropertyService::ReadOnlyProperty,
              CORBA::SystemException);

    virtual void define_properties(const Properties&)
	throw(CosPropertyService::MultipleExceptions,
              CORBA::SystemException);

    virtual CORBA::ULong get_number_of_properties()
	throw(CORBA::SystemException);

    virtual void get_all_property_names(CORBA::ULong, PropertyNames_out,
					PropertyNamesIterator_out)
	throw(CORBA::SystemException);

    virtual CORBA::Any* get_property_value(const char*)
	throw(CosPropertyService::PropertyNotFound,
              CosPropertyService::InvalidPropertyName,
              CORBA::SystemException);

    virtual CORBA::Boolean get_properties(const PropertyNames&,
					  Properties_out)
	throw(CORBA::SystemException);

    virtual void get_all_properties(CORBA::ULong, Properties_out,
				    PropertiesIterator_out)
	throw(CORBA::SystemException);

    virtual void delete_property(const char*)
	throw(CosPropertyService::PropertyNotFound,
	      CosPropertyService::InvalidPropertyName,
	      CosPropertyService::FixedProperty,
	      CORBA::SystemException);

    virtual void delete_properties(const PropertyNames&)
	throw(CosPropertyService::MultipleExceptions,
              CORBA::SystemException);

    virtual CORBA::Boolean delete_all_properties()
	throw(CORBA::SystemException);

    virtual CORBA::Boolean is_property_defined(const char*)
	throw(CosPropertyService::InvalidPropertyName,
              CORBA::SystemException);

    virtual void get_allowed_property_types(PropertyTypes_out)
	throw(CORBA::SystemException);

    virtual void get_allowed_properties(PropertyDefs_out)
	throw(CORBA::SystemException);

    virtual void define_property_with_mode(const char*, const CORBA::Any&,
					   PropertyModeType)
	throw(CosPropertyService::InvalidPropertyName,
              CosPropertyService::ConflictingProperty,
              CosPropertyService::UnsupportedTypeCode,
              CosPropertyService::UnsupportedProperty,
              CosPropertyService::UnsupportedMode,
              CosPropertyService::ReadOnlyProperty,
              CORBA::SystemException);

    virtual void define_properties_with_modes( const PropertyDefs&)
	throw(CosPropertyService::MultipleExceptions,
              CORBA::SystemException);

    virtual PropertyModeType get_property_mode(const char*)
	throw(CosPropertyService::PropertyNotFound,
              CosPropertyService::InvalidPropertyName,
              CORBA::SystemException);

    virtual CORBA::Boolean get_property_modes(const PropertyNames&,
					      PropertyModes_out)
	throw(CORBA::SystemException);

    virtual void set_property_mode(const char*, PropertyModeType)
	throw(CosPropertyService::InvalidPropertyName,
              CosPropertyService::PropertyNotFound,
              CosPropertyService::UnsupportedMode,
              CORBA::SystemException);

    virtual void set_property_modes(const PropertyModes&)
	throw(CosPropertyService::MultipleExceptions,
	      CORBA::SystemException);
};

class PropertyNamesIterator_impl
    : virtual public POA_CosPropertyService::PropertyNamesIterator
{
    PortableServer::POA_var poa_;
    CORBA::ULong idx_;
    PropertyNames names_;

public:

    PropertyNamesIterator_impl(PortableServer::POA_ptr, const PropertyNames&);

    PortableServer::POA_ptr _default_POA()
	{ return PortableServer::POA::_duplicate(poa_); }

    //
    // Standard IDL to C++ Mapping
    //
    virtual void reset()
	throw(CORBA::SystemException);

    virtual CORBA::Boolean next_one(PropertyName_out)
	throw(CORBA::SystemException);

    virtual CORBA::Boolean next_n(CORBA::ULong, PropertyNames_out)
	throw(CORBA::SystemException);

    virtual void destroy()
	throw(CORBA::SystemException);
};

class PropertiesIterator_impl
    : virtual public POA_CosPropertyService::PropertiesIterator
{
    PortableServer::POA_var poa_;
    CORBA::ULong idx_;
    PropertyDefs properties_;

public:

    PropertiesIterator_impl(PortableServer::POA_ptr, const PropertyDefs&);

    PortableServer::POA_ptr _default_POA()
	{ return PortableServer::POA::_duplicate(poa_); }

    //
    // Standard IDL to C++ Mapping
    //
    virtual void reset()
	throw(CORBA::SystemException);

    virtual CORBA::Boolean next_one(Property_out)
	throw(CORBA::SystemException);

    virtual CORBA::Boolean next_n(CORBA::ULong, Properties_out)
	throw(CORBA::SystemException);

    virtual void destroy()
	throw(CORBA::SystemException);
};

class PropertySetFactory_impl
    : virtual public POA_CosPropertyService::PropertySetFactory
{
    PortableServer::POA_var poa_;

public:

    PropertySetFactory_impl(PortableServer::POA_ptr poa)
	: poa_(PortableServer::POA::_duplicate(poa)) { }

    PortableServer::POA_ptr _default_POA()
	{ return PortableServer::POA::_duplicate(poa_); }

    virtual PropertySet_ptr create_propertyset()
	throw(CORBA::SystemException);

    virtual PropertySet_ptr
    create_constrained_propertyset(const PropertyTypes&, const Properties&)
	throw(CORBA::SystemException);

    virtual PropertySet_ptr
    create_initial_propertyset(const Properties&)
	throw(CORBA::SystemException);
};

class PropertySetDefFactory_impl
    : virtual public POA_CosPropertyService::PropertySetDefFactory
{
    PortableServer::POA_var poa_;

public:

    PropertySetDefFactory_impl(PortableServer::POA_ptr poa)
	: poa_(PortableServer::POA::_duplicate(poa)) { }

    //
    // Standard IDL to C++ Mapping
    //
    virtual PropertySetDef_ptr create_propertysetdef()
    throw(CORBA::SystemException);

    virtual PropertySetDef_ptr
    create_constrained_propertysetdef(const PropertyTypes&,
				      const PropertyDefs&)
	throw(CORBA::SystemException);

    virtual PropertySetDef_ptr
    create_initial_propertysetdef(const PropertyDefs&)
	throw(CORBA::SystemException);
};

} // End of namespace CosPropertyService

#endif
